@@ -1,10 +1,11 @@
+# frozen_string_literal: true
 require "bundler"
 require "bundler/vendored_thor"
 
 module Bundler
   class CLI < Thor
     include Thor::Actions
-    AUTO_INSTALL_CMDS = %w(show binstubs outdated exec open console licenses clean)
+    AUTO_INSTALL_CMDS = %w(show binstubs outdated exec open console licenses clean).freeze
 
     def self.start(*)
       super
@@ -136,7 +137,7 @@ module Bundler
     method_option "frozen", :type => :boolean, :banner =>
       "Do not allow the Gemfile.lock to be updated after this install"
     method_option "full-index", :type => :boolean, :banner =>
-      "Use the rubygems modern index instead of the API endpoint"
+      "Fall back to using the single-file index of all gems"
     method_option "gemfile", :type => :string, :banner =>
       "Use the specified gemfile instead of Gemfile"
     method_option "jobs", :aliases => "-j", :type => :numeric, :banner =>
@@ -169,7 +170,11 @@ module Bundler
     map "i" => "install"
     def install
       require "bundler/cli/install"
+      no_install = Bundler.settings[:no_install]
+      Bundler.settings[:no_install] = false if no_install == true
       Install.new(options.dup).run
+    ensure
+      Bundler.settings[:no_install] = no_install unless no_install.nil?
     end
 
     desc "update [OPTIONS]", "update the current environment"
@@ -179,7 +184,7 @@ module Bundler
       possible versions of the gems in the bundle.
     D
     method_option "full-index", :type => :boolean, :banner =>
-      "Use the rubygems modern index instead of the API endpoint"
+      "Fall back to using the single-file index of all gems"
     method_option "group", :aliases => "-g", :type => :array, :banner =>
       "Update a specific group"
     method_option "jobs", :aliases => "-j", :type => :numeric, :banner =>
@@ -192,6 +197,8 @@ module Bundler
       "Update a specific source (and all gems associated with it)"
     method_option "force", :type => :boolean, :banner =>
       "Force downloading every gem."
+    method_option "ruby", :type => :boolean, :banner =>
+      "Update ruby specified in Gemfile.lock"
     def update(*gems)
       require "bundler/cli/update"
       Update.new(options, gems).run
@@ -240,6 +247,11 @@ module Bundler
     method_option "source", :type => :array, :banner => "Check against a specific source"
     method_option "strict", :type => :boolean, :banner =>
       "Only list newer versions allowed by your Gemfile requirements"
+    method_option "major", :type => :boolean, :banner => "Only list major newer versions"
+    method_option "minor", :type => :boolean, :banner => "Only list minor newer versions"
+    method_option "patch", :type => :boolean, :banner => "Only list patch newer versions"
+    method_option "parseable", :aliases => "--porcelain", :type => :boolean, :banner =>
+      "Use minimal formatting for more parseable output"
     def outdated(*gems)
       require "bundler/cli/outdated"
       Outdated.new(options, gems).run
@@ -265,6 +277,8 @@ module Bundler
     method_option "path", :type => :string, :banner =>
       "Specify a different path than the system default ($BUNDLE_PATH or $GEM_HOME). Bundler will remember this value for future installs on this machine"
     method_option "quiet", :type => :boolean, :banner => "Only output warnings and errors."
+    method_option "frozen", :type => :boolean, :banner =>
+      "Do not allow the Gemfile.lock to be updated after this package operation's install"
     long_desc <<-D
       The package command will copy the .gem files for every gem in the bundle into the
       directory ./vendor/cache. If you then check that directory into your source
@@ -356,7 +370,7 @@ module Bundler
     end
 
     desc "gem GEM [OPTIONS]", "Creates a skeleton for creating a rubygem"
-    method_option :bin, :type => :boolean, :default => false, :aliases => "-b", :desc => "Generate a binary for your library."
+    method_option :exe, :type => :boolean, :default => false, :aliases => ["--bin", "-b"], :desc => "Generate a binary executable for your library."
     method_option :coc, :type => :boolean, :desc => "Generate a code of conduct file. Set a default with `bundle config gem.coc true`."
     method_option :edit, :type => :string, :aliases => "-e", :required => false, :banner => "EDITOR",
                          :lazy_default => [ENV["BUNDLER_EDITOR"], ENV["VISUAL"], ENV["EDITOR"]].find {|e| !e.nil? && !e.empty? },
@@ -407,6 +421,8 @@ module Bundler
       "print the lockfile to STDOUT instead of writing to the file system"
     method_option "lockfile", :type => :string, :default => nil, :banner =>
       "the path the lockfile should be written to"
+    method_option "full-index", :type => :boolean, :default => false, :banner =>
+      "Fall back to using the single-file index of all gems"
     def lock
       require "bundler/cli/lock"
       Lock.new(options).run
@@ -417,6 +433,43 @@ module Bundler
       Env.new.write($stdout)
     end
 
+    # Reformat the arguments passed to bundle that include a --help flag
+    # into the corresponding `bundle help #{command}` call
+    def self.reformatted_help_args(args)
+      bundler_commands = all_commands.keys
+      help_flags = %w(--help -h)
+      exec_commands = %w(exec e)
+      help_used = args.select {|a| help_flags.include? a }
+      exec_used = args.select {|a| exec_commands.include? a }
+      command = args.select {|a| bundler_commands.include? a }
+      if exec_used.any? && help_used.any?
+        regex = /
+            ( # Matches `exec --help` or `exec --help foo`
+              (#{exec_commands.join("|")})
+              \s
+              (#{help_flags.join("|")})
+              (.+)*
+            )|
+            ( # Matches `--help exec` or `--help exec foo`
+              (#{help_flags.join("|")})
+              \s
+              (#{exec_commands.join("|")})
+              (.+)*
+            )
+          /x
+        arg_str = args.join(" ")
+        if arg_str =~ regex
+          %w(help exec)
+        else
+          args
+        end
+      elsif command.empty?
+        abort("Could not find command \"#{args.join(" ")}\".")
+      else
+        ["help", command.first]
+      end
+    end
+
   private
 
     # Automatically invoke `bundle install` and resume if
